<?php

declare(strict_types=1);

namespace Drupal\webform_publish\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Provides a my webforms block.
 *
 * @Block(
 *   id = "webform_publish_my_webforms",
 *   admin_label = @Translation("My Webforms"),
 *   category = @Translation("Webform"),
 * )
 */
final class MyWebformsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly AccountProxyInterface $currentUser,
    private readonly EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $webforms = $this->getWebforms();
    if (empty($webforms)) {
      return [
        '#markup' => $this->t('No webforms found.'),
      ];
    }

    $rows = [];
    foreach ($webforms as $webformId => $webform) {
      $settings = $webform->get('third_party_settings');
      $published = FALSE;
      if (isset($settings['webform_publish']) && isset($settings['webform_publish']['published']) && !$settings['webform_publish']['published']) {
        $status = $this->t('Not published');
      }
      else {
        $status = $this->t('Published');
        $published = TRUE;
      }
      $rows[] = [
        'data' => [
          $published ? Link::createFromRoute($webform->get('title'), 'entity.webform.canonical', ['webform' => $webform->get('id')]) : $webform->get('title'),
          $status,
          Link::createFromRoute(
            $this->t('Edit'),
            'entity.webform.edit_form',
            ['webform' => $webform->get('id')],
            [
              'attributes' => [
                'arai-label' => [
                  $this->t('Edit @title', ['@title' => $webform->get('title')]),
                ],
              ],
            ]
          ),
        ],
      ];
    }

    $build['content'] = [
      '#theme' => 'table',
      '#header' => [
        $this->t('Webform'),
        $this->t('Status'),
        $this->t('Edit'),
      ],
      '#rows' => $rows,
    ];
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResult {
    return $account->hasPermission('access own webform configuration') ? AccessResult::allowed() : AccessResult::forbidden();
  }

  protected function getWebforms(): array {
    $webforms = [];
    $uid = $this->currentUser->id();
    $webformIds = $this->configFactory->listAll('webform.webform.');

    foreach ($webformIds as $webformId) {
      $webformConfig = $this->configFactory->get($webformId);
      if ($webformConfig->get('uid') != $uid) {
        continue;
      }
      if ($webformConfig->get('template') == false) {
        $webforms[$webformId] = $webformConfig;
      }
    }
    return $webforms;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    $cacheTags = parent::getCacheTags();
    $cacheTags[] = 'webform';
    return $cacheTags;
  }
}
